@@ -1,30 +1,44 @@
-"""Stores API connection session and sends requests"""
+"""Stores API session and sends requests"""
 
 import re
 import time
+
+import string
 import secrets
-import logging
+
 from functools import partial
 
 from typing import Optional
-from typing import Dict, Any
+from typing import List, Dict, Any
 
 import requests
 
 from cloudscraper import CloudScraper
 
+from .atlog import log, is_debug
+
 from . import atjsparse
 from .aterrors import TokenError
 from .aterrors import CloudflareError
 from .aterrors import AternosPermissionError
 
+
+BASE_URL = 'https://aternos.org'
+AJAX_URL = f'{BASE_URL}/ajax'
+
 REQUA = \
     'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 ' \
     '(KHTML, like Gecko) Chrome/99.0.4844.84 Safari/537.36 OPR/85.0.4341.47'
 
+ARROW_FN_REGEX = r'\(\(\).*?\)\(\);'
+SCRIPT_TAG_REGEX = (
+    rb'<script type=([\'"]?)text/javascript\1>.+?</script>'
+)
+
+SEC_ALPHABET = string.ascii_lowercase + string.digits
+
 
 class AternosConnect:
-
     """Class for sending API requests,
     bypassing Cloudflare and parsing responses"""
 
@@ -34,36 +48,9 @@ class AternosConnect:
         self.session = self.cf_init()
         self.sec = ''
         self.token = ''
-
-    def add_args(self, **kwargs) -> None:
-
-        """Pass arguments to CloudScarper
-        session object __init__
-        if kwargs is not empty
-
-        Args:
-            **kwargs: Keyword arguments
-        """
-
-        if len(kwargs) < 1:
-            logging.debug('**kwargs is empty')
-            return
-
-        logging.debug('New args for CloudScraper: %s', kwargs)
-        self.cf_init = partial(CloudScraper, **kwargs)
-        self.refresh_session()
-
-    def clear_args(self) -> None:
-
-        """Clear CloudScarper object __init__ arguments
-        which was set using add_args method"""
-
-        logging.debug('Creating session object with no keywords')
-        self.cf_init = partial(CloudScraper)
-        self.refresh_session()
+        self.atcookie = ''
 
     def refresh_session(self) -> None:
-
         """Creates a new CloudScraper
         session object and copies all cookies.
         Required for bypassing Cloudflare"""
@@ -74,13 +61,10 @@ class AternosConnect:
         del old_cookies
 
     def parse_token(self) -> str:
-
         """Parses Aternos ajax token that
         is needed for most requests
 
         Raises:
-            RuntimeWarning: If the parser can not
-                find `<head>` tag in HTML response
             TokenError: If the parser is unable
                 to extract ajax token from HTML
 
@@ -89,7 +73,7 @@ class AternosConnect:
         """
 
         loginpage = self.request_cloudflare(
-            'https://aternos.org/go/', 'GET'
+            f'{BASE_URL}/go/', 'GET'
         ).content
 
         # Using the standard string methods
@@ -101,26 +85,44 @@ class AternosConnect:
         # Some checks
         if headtag < 0 or headend < 0:
             pagehead = loginpage
-            raise RuntimeWarning(
+            log.warning(
                 'Unable to find <head> tag, parsing the whole page'
             )
 
-        # Extracting <head> content
-        headtag = headtag + len(head)
-        pagehead = loginpage[headtag:headend]
+        else:
+            # Extracting <head> content
+            headtag = headtag + len(head)
+            pagehead = loginpage[headtag:headend]
+
+        js_code: Optional[List[Any]] = None
 
         try:
             text = pagehead.decode('utf-8', 'replace')
-            js_code = re.findall(r'\(\(\)(.*?)\)\(\);', text)
+            js_code = re.findall(ARROW_FN_REGEX, text)
 
             token_func = js_code[0]
             if len(js_code) > 1:
                 token_func = js_code[1]
 
-            ctx = atjsparse.exec_js(token_func)
-            self.token = ctx.window['AJAX_TOKEN']
+            js = atjsparse.get_interpreter()
+            js.exec_js(token_func)
+            self.token = js['AJAX_TOKEN']
 
         except (IndexError, TypeError) as err:
+
+            log.warning('---')
+            log.warning('Unable to parse AJAX_TOKEN!')
+            log.warning('Please, insert the info below')
+            log.warning('to the GitHub issue description:')
+            log.warning('---')
+
+            log.warning('JavaScript: %s', js_code)
+            log.warning(
+                'All script tags: %s',
+                re.findall(SCRIPT_TAG_REGEX, pagehead)
+            )
+            log.warning('---')
+
             raise TokenError(
                 'Unable to parse TOKEN from the page'
             ) from err
@@ -128,7 +130,6 @@ class AternosConnect:
         return self.token
 
     def generate_sec(self) -> str:
-
         """Generates Aternos SEC token which
         is also needed for most API requests
 
@@ -136,8 +137,8 @@ class AternosConnect:
             Random SEC `key:value` string
         """
 
-        randkey = secrets.token_hex(8)
-        randval = secrets.token_hex(8)
+        randkey = self.generate_sec_part()
+        randval = self.generate_sec_part()
         self.sec = f'{randkey}:{randval}'
         self.session.cookies.set(
             f'ATERNOS_SEC_{randkey}', randval,
@@ -146,6 +147,14 @@ class AternosConnect:
 
         return self.sec
 
+    def generate_sec_part(self) -> str:
+        """Generates a part for SEC token"""
+
+        return ''.join(
+            secrets.choice(SEC_ALPHABET)
+            for _ in range(11)
+        ) + ('0' * 5)
+
     def request_cloudflare(
             self, url: str, method: str,
             params: Optional[Dict[Any, Any]] = None,
@@ -153,8 +162,8 @@ class AternosConnect:
             headers: Optional[Dict[Any, Any]] = None,
             reqcookies: Optional[Dict[Any, Any]] = None,
             sendtoken: bool = False,
-            retry: int = 5) -> requests.Response:
-
+            retries: int = 5,
+            timeout: int = 4) -> requests.Response:
         """Sends a request to Aternos API bypass Cloudflare
 
         Args:
@@ -168,8 +177,9 @@ class AternosConnect:
                 Cookies only for this request
             sendtoken (bool, optional): If the ajax and SEC token
                 should be sent
-            retry (int, optional): How many times parser must retry
+            retries (int, optional): How many times parser must retry
                 connection to API bypass Cloudflare
+            timeout (int, optional): Request timeout in seconds
 
         Raises:
             CloudflareError: When the parser has exceeded retries count
@@ -179,9 +189,14 @@ class AternosConnect:
             API response
         """
 
-        if retry <= 0:
+        if retries <= 0:
             raise CloudflareError('Unable to bypass Cloudflare protection')
 
+        try:
+            self.atcookie = self.session.cookies['ATERNOS_SESSION']
+        except KeyError:
+            pass
+
         self.refresh_session()
 
         params = params or {}
@@ -200,42 +215,45 @@ class AternosConnect:
             headers['X-Requested-With'] = 'XMLHttpRequest'
 
         # requests.cookies.CookieConflictError bugfix
-        reqcookies['ATERNOS_SESSION'] = self.atsession
+        reqcookies['ATERNOS_SESSION'] = self.atcookie
         del self.session.cookies['ATERNOS_SESSION']
 
-        reqcookies_dbg = {
-            k: str(v or '')[:3]
-            for k, v in reqcookies.items()
-        }
+        if is_debug():
 
-        session_cookies_dbg = {
-            k: str(v or '')[:3]
-            for k, v in self.session.cookies.items()
-        }
+            reqcookies_dbg = {
+                k: str(v or '')[:3]
+                for k, v in reqcookies.items()
+            }
 
-        logging.debug('Requesting(%s)%s', method, url)
-        logging.debug('headers=%s', headers)
-        logging.debug('params=%s', params)
-        logging.debug('data=%s', data)
-        logging.debug('req-cookies=%s', reqcookies_dbg)
-        logging.debug('session-cookies=%s', session_cookies_dbg)
+            session_cookies_dbg = {
+                k: str(v or '')[:3]
+                for k, v in self.session.cookies.items()
+            }
+
+            log.debug('Requesting(%s)%s', method, url)
+            log.debug('headers=%s', headers)
+            log.debug('params=%s', params)
+            log.debug('data=%s', data)
+            log.debug('req-cookies=%s', reqcookies_dbg)
+            log.debug('session-cookies=%s', session_cookies_dbg)
 
         if method == 'POST':
             sendreq = partial(
                 self.session.post,
                 params=params,
-                data=data
+                data=data,
             )
         else:
             sendreq = partial(
                 self.session.get,
-                params={**params, **data}
+                params={**params, **data},
             )
 
         req = sendreq(
             url,
             headers=headers,
-            cookies=reqcookies
+            cookies=reqcookies,
+            timeout=timeout,
         )
 
         resp_type = req.headers.get('content-type', '')
@@ -243,17 +261,17 @@ class AternosConnect:
         cloudflare = req.status_code == 403
 
         if html_type and cloudflare:
-            logging.info('Retrying to bypass Cloudflare')
-            time.sleep(0.2)
+            log.info('Retrying to bypass Cloudflare')
+            time.sleep(0.3)
             return self.request_cloudflare(
                 url, method,
                 params, data,
                 headers, reqcookies,
-                sendtoken, retry - 1
+                sendtoken, retries - 1
             )
 
-        logging.debug('AternosConnect received: %s', req.text[:65])
-        logging.info(
+        log.debug('AternosConnect received: %s', req.text[:65])
+        log.info(
             '%s completed with %s status',
             method, req.status_code
         )
@@ -266,7 +284,6 @@ class AternosConnect:
 
     @property
     def atsession(self) -> str:
-
         """Aternos session cookie,
         empty string if not logged in
 
