@@ -7,6 +7,7 @@ from typing import TYPE_CHECKING
 
 import lxml.html
 
+from .atconnect import BASE_URL, AJAX_URL
 if TYPE_CHECKING:
     from .atserver import AternosServer
 
@@ -32,7 +33,6 @@ class PlayersList:
             self,
             lst: Union[str, Lists],
             atserv: 'AternosServer') -> None:
-
         """Class for managing operators,
         whitelist and banned players lists
 
@@ -50,8 +50,8 @@ class PlayersList:
         # whl_je = whitelist for java
         # whl_be = whitelist for bedrock
         # whl = common whitelist
-        common_whl = (self.lst == Lists.whl)
-        bedrock = (atserv.is_bedrock)
+        common_whl = self.lst == Lists.whl
+        bedrock = atserv.is_bedrock
 
         if common_whl and bedrock:
             self.lst = Lists.whl_be
@@ -60,7 +60,6 @@ class PlayersList:
         self.parsed = False
 
     def list_players(self, cache: bool = True) -> List[str]:
-
         """Parse a players list
 
         Args:
@@ -75,7 +74,7 @@ class PlayersList:
             return self.players
 
         listreq = self.atserv.atserver_request(
-            f'https://aternos.org/players/{self.lst.value}',
+            f'{BASE_URL}/players/{self.lst.value}',
             'GET'
         )
         listtree = lxml.html.fromstring(listreq.content)
@@ -93,7 +92,6 @@ class PlayersList:
         return result
 
     def add(self, name: str) -> None:
-
         """Appends a player to the list by the nickname
 
         Args:
@@ -101,7 +99,7 @@ class PlayersList:
         """
 
         self.atserv.atserver_request(
-            'https://aternos.org/panel/ajax/players/add.php',
+            f'{AJAX_URL}/players/add.php',
             'POST', data={
                 'list': self.lst.value,
                 'name': name
@@ -111,7 +109,6 @@ class PlayersList:
         self.players.append(name)
 
     def remove(self, name: str) -> None:
-
         """Removes a player from the list by the nickname
 
         Args:
@@ -119,7 +116,7 @@ class PlayersList:
         """
 
         self.atserv.atserver_request(
-            'https://aternos.org/panel/ajax/players/remove.php',
+            f'{AJAX_URL}/players/remove.php',
             'POST', data={
                 'list': self.lst.value,
                 'name': name
